@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2012 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2014 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -77,8 +77,8 @@ def main():
         # Send this with a 404 status.
         print 'Status: 404 Not Found'
         admin_overview(_('No such list <em>%(safelistname)s</em>'))
-        syslog('error', 'admin.py access for non-existent list: %s',
-               listname)
+        syslog('error', 'admin: No such list "%s": %s\n',
+               listname, e)
         return
     # Now that we know what list has been requested, all subsequent admin
     # pages are shown in that list's preferred language.
@@ -88,7 +88,8 @@ def main():
 
     # CSRF check
     safe_params = ['VARHELP', 'adminpw', 'admlogin',
-                   'letter', 'chunk', 'findmember']
+                   'letter', 'chunk', 'findmember',
+                   'legend']
     params = cgidata.keys()
     if set(params) - set(safe_params):
         csrf_checked = csrf_check(mlist, cgidata.getvalue('csrf_token'))
@@ -1151,7 +1152,8 @@ def membership_options(mlist, subcat, cgidata, doc, form):
                 continue
             start = chunkmembers[i*chunksz]
             end = chunkmembers[min((i+1)*chunksz, last)-1]
-            link = Link(url + 'chunk=%d' % i, _('from %(start)s to %(end)s'))
+            link = Link(url + 'chunk=%d' % i + findfrag,
+                        _('from %(start)s to %(end)s'))
             buttons.append(link)
         buttons = UnorderedList(*buttons)
         container.AddItem(footer + buttons.Format() + '<p>')
@@ -1436,10 +1438,12 @@ def change_options(mlist, category, subcat, cgidata, doc):
         removals += cgidata['unsubscribees_upload'].value
     if removals:
         names = filter(None, [n.strip() for n in removals.splitlines()])
-        send_unsub_notifications = int(
-            cgidata['send_unsub_notifications_to_list_owner'].value)
-        userack = int(
-            cgidata['send_unsub_ack_to_this_batch'].value)
+        send_unsub_notifications = safeint(
+            'send_unsub_notifications_to_list_owner',
+            mlist.admin_notify_mchanges)
+        userack = safeint(
+            'send_unsub_ack_to_this_batch',
+            mlist.send_goodbye_msg)
         unsubscribe_errors = []
         unsubscribe_success = []
         for addr in names:
@@ -1463,11 +1467,7 @@ def change_options(mlist, category, subcat, cgidata, doc):
             doc.AddItem('<p>')
     # See if this was a moderation bit operation
     if cgidata.has_key('allmodbit_btn'):
-        val = cgidata.getvalue('allmodbit_val')
-        try:
-            val = int(val)
-        except VallueError:
-            val = None
+        val = safeint('allmodbit_val')
         if val not in (0, 1):
             doc.addError(_('Bad moderation flag value'))
         else:
